/////////////////////////////////////////////////////////////////////
// Internal engine functions
//
// This script is used when the engine runs a level or model.
// 
// (c) jcl / Conitec  2006
/////////////////////////////////////////////////////////////////////
// Apr 2008 - minor change: automatic capturing to BMP format instead of JPG
/////////////////////////////////////////////////////////////////////
#ifndef default_c
#define default_c
#include <acknex.h>  // standard C-Script objects

var def_dfps;
var def_dtps;
var def_dtlv;
var def_dtcs;
var def_dtac;
ANGLE def_cang;

VIEW* map_view = 
{
	flags = WIREFRAME + NOCULL; 
	layer = 1; 
	roll = 270; 
	tilt = -90; 
	arc = 10; 
}

PANEL* def_debug_pan = 
{
    pos_x = 4; 
	pos_y = 12; 
	layer = 998; 

	digits(0,2,"%4.0fx",*,1,screen_size.x);
	digits(30,2,"%.0f",*,1,screen_size.y);
    digits(0,12,"fps%5.0f",*,16,def_dfps);
    digits(0,22,"pps%5.0fk",*,0.016,def_dtps);
	digits(0,32,"x%7.0f",*,1,camera.x);
    digits(0,42,"y%7.0f",*,1,camera.y);
    digits(0,52,"z%7.0f",*,1,camera.z);
    digits(0,62,"pan%5.0f",*,1,def_cang.pan);
    digits(0,72,"til%5.0f",*,1,def_cang.tilt);
    digits(0,82,"rol%5.0f",*,1,def_cang.roll);
//    digits(0,92,"sco%7.3f",*,1,scope);

    digits(65,2,"ms/frame",*,0,0);
    digits(65,22,"ent%5.1f",*,1,time_entities);
    digits(65,12,"geo%5.1f",*,1,def_dtlv);
    digits(65,32,"par%5.1f",*,1,time_effects);
    digits(65,42,"mir%5.1f",*,1,time_portals);
    digits(65,52,"pan%5.1f",*,1,time_panels);
    digits(65,62,"ref%5.1f",*,1,time_update);
    digits(65,72,"srv%5.1f",*,1,def_dtcs);
    digits(65,82,"fnc%5.1f",*,1,def_dtac);
    digits(65,92,"phy%5.1f",*,1,time_physics);

    digits(130,2,"count of",*,0,0;
    digits(130,12,"ent%5.0f",*,1,num_entities);
    digits(130,22,"vis%5.0f",*,1,num_visents);
    digits(130,32,"tri%5.0fk",*,0.001,num_vistriangles);
    digits(130,42,"par%5.0f",*,1,num_particles);
    digits(130,52,"lit%5.0f",*,1,num_lights);
    digits(130,62,"med%5.0f",*,1,num_mediastreams);
    digits(130,72,"fnc%5.0f",*,1,num_actions);
    digits(130,82,"bdy%5.0f",*,1,ph_num_bodies);
    digits(130,92,"con%5.0f",*,1,ph_num_contacts);

    digits(200,2,"memory kb",*,0,0);
    digits(200,12,"nex%6.0f",*,1,nexus);
    digits(200,22,"geo%6.0f",*,1,d3d_texsurfs);
    digits(200,32,"sha%6.0f",*,1,d3d_texsmaps);
    digits(200,42,"ent%6.0f",*,1,d3d_texskins);
    digits(200,52,"bmp%6.0f",*,1,d3d_texbmaps);
    digits(200,62,"fr%7.0f",*,1,d3d_texfree);

    digits(270,2,"network",*,0,0);
    digits(270,12,"lat%4.0f",*,1,dplay_latency);
    digits(270,22,"bps%4.0f",*,1,dplay_bps);
    digits(270,32,"bpk%4.0f",*,1,dplay_bpspeak);
    digits(270,42,"rel%4.0f",*,1,dplay_bpsreliable);
    digits(270,52,"unr%4.0f",*,1,dplay_bpsunreliable);
    digits(270,62,"drp%4.0f",*,1,dplay_dropped);

}

var def_oldmouse = 0;

void def_debug() 
{
    beep();
    if (key_shift) 
	{
        freeze_mode = !freeze_mode;
        if (freeze_mode) {
            def_oldmouse = mouse_mode;
            mouse_mode = 2;
            mouse_pointer = 2;
        } else {
	        mouse_mode = def_oldmouse;
        }
        while (freeze_mode) {
            mouse_pos.x = mouse_cursor.x;
            mouse_pos.y = mouse_cursor.y;
            if (mouse_left)
				watched = mouse_ent;
            wait(1);
        }
        return;
    }

    if (key_ctrl) {
		toggle(map_view,VISIBLE);
		while is(map_view,VISIBLE) {
			vec_set(&map_view.x,&camera.x);
			map_view.z += 10000;
			wait(1);
		}
		return;
    }

    if (d3d_lines) {
            reset(def_debug_pan,VISIBLE);
            d3d_lines = 0;
			diag_mode = 0;
    } else {
        if is(def_debug_pan,VISIBLE) {
                d3d_lines = 1;
        } else {
            set(def_debug_pan,VISIBLE);
			diag_mode = 1;
            while is(def_debug_pan,VISIBLE) {
				def_dfps = 0.9*def_dfps+0.1/time_frame;
				def_dtps = 0.9*def_dtps+0.1*num_vistriangles/time_frame;
				def_dtlv = time_level+time_sky;
				def_dtcs = time_client+time_server+time_idle;
				def_dtac = time_actions+time_pfuncs;
				def_cang.pan = cycle(camera.pan,0,360); 
				def_cang.tilt = cycle(camera.tilt,-180,180); 
				def_cang.roll = cycle(camera.roll,-180,180); 
				wait(1);
            }
        }
    }
}

void def_exit() 
{
	if (!key_f4 || key_alt)
		sys_exit(NULL); 
}

var def_shot_num = 0;
void def_shot() 
{ 
	file_for_screen("shot_.bmp",def_shot_num); 
	beep(); 
	def_shot_num++; 
}

void def_save() 
{ 
	game_save(app_name,1,SV_ALL-SV_INFO); 
	beep(); 
}

void def_load() 
{ 
	game_load(app_name,1); 
}

void def_video() 
{
	var mode = video_mode;
	while(1) {
		if (!key_shift) 
			mode++; 
		else 
			mode--;
		mode = cycle(mode,6,12); 
		if (video_switch(mode,0,0)) 
			break;
   }
}

void def_screen() 
{
	if (!key_alt) 
		return;
	var mode = video_screen + 1;
	if (mode > 2) 
		mode = 1;
	if (video_switch(0,0,mode) == 0) 
		beep();
}

var def_mvol = 50;
var def_svol = 50;
void def_sound() 
{
    if (sound_vol==0) { 
		sound_vol = def_svol; 
		midi_vol = def_mvol; 
		return; 
	}
    if (midi_vol==0) { 
		def_svol = sound_vol; 
		sound_vol = 0; 
		return; 
	}
    def_mvol = midi_vol; 
	midi_vol = 0;
}


var def_camera = 0;
VECTOR* def_cam_dist = { x=100; y=0; z=100; }

void def_move()
{
	VECTOR force;    // ahead force
	ANGLE aforce;    // angular force
	VECTOR speed; 
	ANGLE aspeed; 
	VECTOR dist; 

// initialize speed and distance
	vec_zero(speed);
	vec_zero(aspeed);
	vec_zero(dist);

	if (1 < run_mode && run_mode < 5) 
		def_camera = 2;	// prevent player movement in entity viewer mode

	while (def_camera) 
	{
        aforce.tilt = 5*(key_pgup - key_pgdn + mouse_force.y);
        if (key_alt==0) {
            aforce.pan = -5*(key_force.x + mouse_force.x + joy_force.x);
            aforce.roll = 0;
        } else {
            aforce.pan = 0;
            aforce.roll = 5*(key_force.x + mouse_force.x + joy_force.x);
        }

		vec_add(camera.pan,vec_accelerate(dist,aspeed,aforce,0.8));

        force.x = 7*(key_force.y + joy_force.y + mouse_right + key_w - key_s);
        force.y = 3*(key_comma - key_period + key_a - key_d);
        force.z = 3*(key_home - key_end);
        vec_accelerate(dist,speed,force,0.5);

        if (NULL != player && 1 == def_camera) {
            c_move(player,dist,nullvector,IGNORE_PASSABLE|IGNORE_PASSENTS|GLIDE);
            camera.genius = player;
            vec_set(player.pan,camera.pan);
            vec_set(camera.x,def_cam_dist);
            vec_rotate(camera.x,camera.pan);
            vec_add(camera.x,player.x);
        } else {
            camera.genius = NULL;
            vec_add(camera.x,vec_rotate(dist,camera.pan));
        }
        wait(1);
    }
}

void def_moveset() 
{
    def_camera += 1;
	if (NULL != player)
		def_camera = cycle(def_camera,0,3);	// 0-1-2
	else
		def_camera = cycle(def_camera,0,2);	// 0-1
    if (!key_shift && def_camera > 0) {
        beep();
		def_move();
    }
}

TEXT* def_ctxt = { string("Enter var or STRING","#80"); layer = 999; }

void def_console() // Tab
{
   def_ctxt.pos_x = 2;
   def_ctxt.pos_y = screen_size.y-30;
   toggle(def_ctxt,VISIBLE);
   while is(def_ctxt,VISIBLE) {
     beep();
     inkey((def_ctxt.pstring)[1]);
	 if (13 == result) {
		void* found = var_for_name((def_ctxt.pstring)[1]);
		if (!found) execute((def_ctxt.pstring)[1]);
	 } else
		reset(def_ctxt,VISIBLE);
   }
}

TEXT* def_dtxt = {
   pos_x = 2; pos_y = 2; layer = 999;
   string = debug_str,watch_str;
   flags = VISIBLE;
}

void def_startup() 
{ 
	if (!on_f2) on_f2 = def_save;
	if (!on_f3) on_f3 = def_load;
	if (!on_f5) on_f5 = def_video;
	if (!on_f4) on_f4 = def_exit;
	if (!on_f6) on_f6 = def_shot;
	if (!on_f11) on_f11 = def_debug;
	if (!on_f12) on_f12 = def_sound;

	if (!on_0) on_0 = def_moveset;
	if (!on_enter) on_enter = def_screen;
	if (!on_tab) on_tab = def_console;
	if (!on_esc) on_esc = def_exit;
	if (!on_close) on_close = def_exit;

// activate default movement and settings in entity viewer mode
	if (1 < run_mode && run_mode < 5) {
		d3d_autotransparency = ON;
		terrain_lod = 4;
		def_camera = 2;
		def_move();
	}

}

#endif